﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.Net.Sockets;
using System.ComponentModel;
using SimplyMessengerLibrary;
using Xceed.Wpf.Toolkit;

namespace SimplyMessenger
{
    /// <summary>
    /// Logique d'interaction pour MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        const int portNo = 500;
        TcpClient client;
        byte[] data;

        private string TColor;

        private ObservableCollection<Message> MessageCollection
        {
            get
            {
                return (ObservableCollection<Message>)GetValue(MessageCollectionProperty);
            }
            set
            {
                SetValue(MessageCollectionProperty, value);
            }
        }

        // Using a DependencyProperty as the backing store for TextCollection.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty MessageCollectionProperty =
            DependencyProperty.Register("MessageCollection", typeof(ObservableCollection<Message>), typeof(MainWindow), new UIPropertyMetadata(null));

        public MainWindow()
        {
            FileHelper fh = new FileHelper();
            fh.EraseAllTxtFiles();
            this.MessageCollection = new ObservableCollection<Message>();
            //this.TextCollection.Add("Connard");

            

            this.DataContext = this;
            InitializeComponent();

            this.ColorPickerControl.SelectedColor = Color.FromRgb(0, 0, 0);
        }

        private void refresh()
        { }

        private string MessageToString(Message message)
        {
            try
            {
                IPHelper iph = new IPHelper();
                SerializationHelper sh = new SerializationHelper();
                string path = "Client_" + iph.getAddr() + ".txt";
                sh.Serialization(path, message);
                return path;
            }
            catch (Exception ex)
            {
                System.Windows.MessageBox.Show(this, ex.ToString());
                return null;
            }
        }
        
        public void SendMessage(String messagePath)
        {
            if (messagePath == null)
                return;
            try
            {
                FileHelper fh = new FileHelper();
                string xmlMessage = fh.FileRead(messagePath);
                //---send a message to the server
                NetworkStream ns = client.GetStream();
                byte[] data = System.Text.Encoding.Unicode.GetBytes(xmlMessage);
                //---send the text---
                ns.Write(data, 0, data.Length);
                ns.Flush();
            }
            catch (Exception ex)
            { System.Windows.MessageBox.Show(this, ex.ToString()); }
        }

        public void ReceiveMessage(IAsyncResult ar)
        {
            try
            {
                int bytesRead = 0;
                bytesRead = client.GetStream().EndRead(ar);
                if (bytesRead < 1)
                { return; }
                else
                {
                    //get the xml
                    var xml = System.Text.Encoding.Unicode.GetString(data, 0, bytesRead);

                    //Unserialize the XML
                    FileHelper fh = new FileHelper();
                    SerializationHelper sh = new SerializationHelper();
                    Random rnd = new Random();
                    string path = "Client_" + rnd.Next() + ".txt";
                    //fh.EraseAllTxtFiles(); //Hacking
                    fh.FileWrite(path, xml);
                    Message message = sh.Deserialization(path);
                    //
                    
                    //Message action Reading
                    switch (message.MsgAction)
                    {
                        case "Login":
                            {
                                message.MessageText = message.NickName + " is logging in";
                                message.NickName = "Server";
                                message.TColor = "#FF0000";
                                break;
                            }
                        case "Text":
                            {

                                break;
                            }
                    }
                    //

                    Application.Current.Dispatcher.Invoke(new Action(() => MessageCollection.Add(message)));

                    //thread = new System.Threading.Thread(
                    //        new System.Threading.ThreadStart(
                    //            delegate()
                    //            {
                    //                MainWindow wind = App.Current.MainWindow as MainWindow;
                    //                wind.Dispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Normal, new Action(() => MessageHistory.ItemsSource = TextCollection ));

                    //            }));
                    refresh();
                }
                client.GetStream().BeginRead(data, 0, Convert.ToInt32(client.ReceiveBufferSize), ReceiveMessage, null);
            }
            catch (Exception
            ex)
            { Console.WriteLine(ex.ToString()); }
        }

        private Message CreateMessage(string messageText, string messageType="Normal", string messageAction="Text")
        {
            
            Message message = new Message();

            message.NickName = this.nicknameBox.Text;

            message.MsgType = messageType;
            
            message.MsgAction = messageAction;
           
            message.TColor = TColor;
            message.Date = DateTime.Now;
            message.MessageText = messageText;
            return message;
        }

        private void btnSend_Click(object sender, RoutedEventArgs e)
        {
            Message message = CreateMessage(this.messageblock.Text);
            SendMessage(MessageToString(message));
            messageblock.Clear();
        }

        //---delegate and subroutine to update the TextBox control
        public delegate void delUpdateHistory(string str);

        public void UpdateHistory(string str)
        { }

        private void btnSignIn_Click(object sender, RoutedEventArgs e)
        {
            if ((string)btnSignIn.Content == "Sign In")
            {
                try
                {
                    Message signInMessage = CreateMessage(this.nicknameBox.Text,"Normal","Login");
                    string path = MessageToString(signInMessage);
                    //---connect to server
                    client = new TcpClient();
                    client.Connect(IPTextBox.Text, portNo);
                    data = new byte[client.ReceiveBufferSize + 1];
                    SendMessage(path);
                    //---read from server
                    client.GetStream().BeginRead(data, 0, Convert.ToInt32(client.ReceiveBufferSize), ReceiveMessage, null);
                    btnSignIn.Content = "Sign Out";
                    IPTextBox.IsEnabled = false;
                    nicknameBox.IsEnabled = false;
                    btnSend.IsEnabled = true;
                }
                catch (Exception ex)
                { System.Windows.MessageBox.Show(this, ex.ToString()); }
            }
            else
            {
                //---disconnect from server
                Disconnect();
                btnSignIn.Content = "Sign In";
                btnSend.IsEnabled = false;
                IPTextBox.IsEnabled = true;
                nicknameBox.IsEnabled = true;
            }
        }

        public void Disconnect()
        {
            //---Disconnect from server
            try
            {
                client.GetStream().Close();
                client.Close();
            }
            catch (Exception ex)
            { System.Windows.MessageBox.Show(this, ex.ToString()); }
        }

        private void ColorPicker_SelectedColorChanged_1(object sender, RoutedPropertyChangedEventArgs<Color> e)
        {
            ColorPicker cp = (ColorPicker)sender;
            TColor = cp.SelectedColor.ToString();
        }
    }
}